<?php
/**
 * @package     CSVI
 * @subpackage  Templates
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\Utilities\ArrayHelper;

// Include the CSVI addon maintenance model file
require_once JPATH_PLUGINS . '/csviaddon/csvi/com_csvi/model/maintenance.php';

/**
 * Templates controller.
 *
 * @package     CSVI
 * @subpackage  Templates
 * @since       6.0
 */
class CsviControllerTemplates extends JControllerAdmin
{
	/**
	 * The prefix to use with controller messages.
	 *
	 * @var    string
	 * @since  7.2.1
	 */
	protected $text_prefix = 'COM_CSVI_TEMPLATES';

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 *
	 * @since   1.6
	 */
	public function __construct($config = [])
	{
		parent::__construct($config);

		$this->registerTask('unfeatured', 'featured');
	}

	/**
	 * Proxy for getModel.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  The array of possible config values. Optional.
	 *
	 * @return  JModel
	 *
	 * @since   6.6.0
	 */
	public function getModel($name = 'Template', $prefix = 'CsviModel', $config = array('ignore_request' => true))
	{
		return parent::getModel($name, $prefix, $config);
	}

	/**
	 * Redirect the user to the template fields view to manage the template fields.
	 *
	 * @return  array  The field option objects.
	 *
	 * @since   6.0
	 */
	public function templateFields()
	{
		$post = $this->input->get('cid', [], 'array');

		if (isset($post[0]))
		{
			$this->setRedirect('index.php?option=com_csvi&view=templatefields&csvi_template_id=' . $post[0]);
		}
		else
		{
			$this->setRedirect('index.php?option=com_csvi&view=templates', Text::_('COM_CSVI_NO_TEMPLATE_SELECTED'));
		}
	}

	/**
	 * Duplicate a template.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function duplicate()
	{
		// Check for request forgeries.
		JSession::checkToken() or jexit(Text::_('JINVALID_TOKEN'));

		/** @var CsviModelTemplate $model */
		$model = $this->getModel();

		try
		{
			$model->createCopy($this->input->get('cid', [], 'array'));

			$this->setRedirect('index.php?option=com_csvi&view=templates', JText::_('COM_CSVI_TEMPLATE_COPIED'));
		}
		catch (Exception $e)
		{
			$this->setRedirect('index.php?option=com_csvi&view=templates', $e->getMessage(), 'error');
		}
	}

	/**
	 * Backup templates.
	 *
	 * @return  void.
	 *
	 * @since   7.12.0
	 */
	public function backupTemplates()
	{
		// Check for request forgeries.
		JSession::checkToken() or jexit(Text::_('JINVALID_TOKEN'));

		/** @var CsviModelTemplate $model */
		$model = $this->getModel();

		try
		{
			$fileLocation = $model->backupTemplates($this->input->get('cid', [], 'array'));

			/** @var CsviModelExports $exportsModel */
			$exportsModel = $this->getModel('Exports', 'CsviModel');

			if ($fileLocation)
			{
				$exportsModel->downloadFile($fileLocation);
			}
		}
		catch (Exception $e)
		{
			$this->setRedirect('index.php?option=com_csvi&view=templates', $e->getMessage(), 'error');
		}
	}

	/**
	 * Set templates as favourite.
	 *
	 * @return  void.
	 *
	 * @since   8.0.0
	 */
	public function featured(): void
	{
		// Check for request forgeries.
		JSession::checkToken() or jexit(Text::_('JINVALID_TOKEN'));

		$ids    = $this->input->get('cid', [], 'array');
		$values = ['featured' => 1, 'unfeatured' => 0];
		$task   = $this->getTask();
		$value  = ArrayHelper::getValue($values, $task, 0, 'int');

		/** @var CsviModelTemplate $model */
		$model = $this->getModel();

		try
		{
			$message = ($value) ? Text::_('COM_CSVI_TEMPLATE_SET_TO_FEATURED') : Text::_('COM_CSVI_TEMPLATE_SET_TO_UNFEATURED');
			$model->featured($ids, $value);
			$this->setRedirect('index.php?option=com_csvi&view=templates', $message);
		}
		catch (Exception $exception)
		{
			$this->setRedirect('index.php?option=com_csvi&view=templates', $exception->getMessage(), 'error');
		}
	}
}
